﻿Imports System.Collections
Namespace System.Drawing
   ''' <summary>a compatibily-Structure for Size and Point</summary>
   <Serializable()> _
   <DebuggerDisplay("{X} {Y}")> _
   Public Structure Vector : Implements IEnumerable(Of Integer)

      Public Shared ReadOnly ControlDefaultSize As New Vector(50, 20)
      Public Shared ReadOnly ControlMinSize As New Vector(20, 13)

      ' both Size and Point can convert to and from Vector.
      ' X,Y also can be accessed by Index 0 or 1 as well as by Index True or False. This avoids Code-
      '°Repeating, because you don't need to implement individual computation-code for horizontal and
      '°vertical purposes - you can use the **same** code, just passing another Index.

      Public X, Y As Integer
      Public Sub New(ByVal x As Integer, ByVal y As Integer)
         Me.X = x : Me.Y = y
      End Sub
      Public Sub New(ByVal eachMember As Integer)
         X = eachMember : Y = X
      End Sub
      Public Shared Function From(ByVal eachMember As Integer) As Vector
         Return New Vector(eachMember)
      End Function
      Public Shared Function From(ByVal x As Integer, ByVal y As Integer) As Vector
         Return New Vector(x, y)
      End Function
      Public Shared Function From(ByVal v As Vector) As Vector
         Return v 'implicit conversion of Point and Size
      End Function
      Public Function Add(ByVal x As Integer, ByVal y As Integer) As Vector
         Return New Vector(Me.X + x, Me.Y + y)
      End Function
      Public Function Add(ByVal amount As Integer) As Vector
         Return New Vector(X + amount, Y + amount)
      End Function
      Public Function Add(ByVal v As Vector) As Vector
         Return New Vector(X + v.X, Y + v.Y)
      End Function
      Public Function Subtract(ByVal x As Integer, ByVal y As Integer) As Vector
         Return New Vector(Me.X - x, Me.Y - y)
      End Function
      Public Function Subtract(ByVal amount As Integer) As Vector
         Return New Vector(X - amount, Y - amount)
      End Function
      Public Function Subtract(ByVal v As Vector) As Vector
         Return New Vector(X - v.X, Y - v.Y)
      End Function
      Default Public Property Item(ByVal vertical As Integer) As Integer
         Get
            If vertical = 0 Then Return X
            Return Y
         End Get
         Set(ByVal value As Integer)
            If vertical = 0 Then X = value Else Y = value
         End Set
      End Property
      Default Public Property Item(ByVal vertical As Boolean) As Integer
         Get
            If vertical Then Return Y
            Return X
         End Get
         Set(ByVal value As Integer)
            If vertical Then Y = value Else X = value
         End Set
      End Property
      ''' <summary>
      ''' return-values: -1: no difference, 0: horizontal diff, 1: vertical diff, 2: diff in both directions
      ''' </summary>
      Public Function DifferenceDirection(ByVal v2 As Vector) As Integer
         Dim ret = 0
         For vertical As Integer = 0 To 1
            If Item(vertical) <> v2(vertical) Then ret = ret Or vertical + 1
         Next
         Return ret - 1
      End Function
      Public Overrides Function ToString() As String
         Return String.Concat("{", X, " ", Y, "}")
      End Function
      Public Function Scale(ByVal x As Double, ByVal y As Double) As Vector
         Return New Vector(CInt(Me.X * x), CInt(Me.Y * y))
      End Function
      Public Function Scale(ByVal value As Double) As Vector
         Return New Vector(CInt(Me.X * value), CInt(Me.Y * value))
      End Function

      Public Function Union(ByVal other As Vector) As Vector
         Return New Vector(Math.Max(X, other.X), Math.Max(Y, other.Y))
      End Function
      Public Function Intersect(ByVal other As Vector) As Vector
         Return New Vector(Math.Min(X, other.X), Math.Min(Y, other.Y))
      End Function
      Public Shared Widening Operator CType(ByVal sz As Size) As Vector
         Return New Vector(sz.Width, sz.Height)
      End Operator
      Public Shared Widening Operator CType(ByVal pt As Point) As Vector
         Return New Vector(pt.X, pt.Y)
      End Operator
      Public Shared Widening Operator CType(ByVal v As Vector) As Point
         Return New Point(v.X, v.Y)
      End Operator
      Public Shared Widening Operator CType(ByVal v As Vector) As Size
         Return New Size(v.X, v.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As Size, ByVal v2 As Vector) As Vector
         Return New Vector(v1.Width + v2.X, v1.Height + v2.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As Vector, ByVal v2 As Vector) As Vector
         Return New Vector(v1.X + v2.X, v1.Y + v2.Y)
      End Operator
      Public Shared Operator +(ByVal v1 As Vector, ByVal sz As Size) As Vector
         Return New Vector(v1.X + sz.Width, v1.Y + sz.Height)
      End Operator
      Public Shared Operator -(ByVal v1 As Size, ByVal v2 As Vector) As Vector
         Return New Vector(v1.Width - v2.X, v1.Height - v2.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As Vector, ByVal v2 As Vector) As Vector
         Return New Vector(v1.X - v2.X, v1.Y - v2.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As Vector) As Vector
         Return New Vector(-v1.X, -v1.Y)
      End Operator
      Public Shared Operator -(ByVal v1 As Vector, ByVal sz As Size) As Vector
         Return New Vector(v1.X - sz.Width, v1.Y - sz.Height)
      End Operator
      Public Shared Operator =(ByVal v1 As Size, ByVal v2 As Vector) As Boolean
         Return v1.Width = v2.X AndAlso v1.Height = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As Size, ByVal v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As Vector, ByVal v2 As Size) As Boolean
         Return v1.X = v2.Width AndAlso v1.Y = v2.Height
      End Operator
      Public Shared Operator <>(ByVal v1 As Vector, ByVal v2 As Size) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As Point, ByVal v2 As Vector) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As Point, ByVal v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As Vector, ByVal v2 As Point) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As Vector, ByVal v2 As Point) As Boolean
         Return Not v1 = v2
      End Operator
      Public Shared Operator =(ByVal v1 As Vector, ByVal v2 As Vector) As Boolean
         Return v1.X = v2.X AndAlso v1.Y = v2.Y
      End Operator
      Public Shared Operator <>(ByVal v1 As Vector, ByVal v2 As Vector) As Boolean
         Return Not v1 = v2
      End Operator
      Public Overrides Function Equals(ByVal obj As Object) As Boolean
         If TypeOf obj Is Vector OrElse TypeOf obj Is Point OrElse TypeOf obj Is Size Then Return Me = CType(obj, Vector)
      End Function
      Public Overrides Function GetHashCode() As Integer
         Return X Xor Y
      End Function
      Public Function EachEquals(ByVal amount As Integer) As Boolean
         Return X = amount AndAlso Y = amount
      End Function
      ''' <summary>enables Linq-power to Vectors Elements</summary>
      Public Function GetEnumerator() As IEnumerator(Of Integer) Implements _
            IEnumerable(Of Integer).GetEnumerator
         Return DirectCast(New Integer() {X, Y}, IEnumerable(Of Integer)) _
            .GetEnumerator
      End Function

      Private Function GetEnumerator1() As IEnumerator Implements _
            IEnumerable.GetEnumerator
         Return DirectCast(New Integer() {X, Y}, IEnumerable(Of Integer)) _
            .GetEnumerator
      End Function
   End Structure

End Namespace
