﻿Imports System.ComponentModel
Imports System.Windows.Forms

Namespace System.Data

   ''' <summary>some common basics of DatasetXmlAdapter and DatasetAdapter</summary>
   Public MustInherit Class DatasetAdapterBase

      Public Shared ReadOnly Attachs As New ComponentProperty(Of DataSet, DatasetAdapterBase)

      Protected _DataSet As DataSet
      Public HasChanges As Func(Of Boolean)
      Public ReadOnly HandleFormClosing As FormClosingEventHandler = AddressOf _HandleFormClosing

      Protected MustOverride Sub _Save(ByVal tbls As List(Of DataTable))
      Public Overridable Property DataSet() As DataSet
         Get
            Return _DataSet
         End Get
         Set(ByVal value As DataSet)
            If _DataSet Is value Then Return
            _DataSet = value
            HasChanges = AddressOf value.HasChanges
         End Set
      End Property

      Public Sub Save(ByVal form As Form, Optional ByVal checkForChanges As Boolean = True)
         Dim tbls = DataSet.GetRankedTables
         For Each tb In tbls
            For Each bs In tb.Attach.BindingSources
               bs.EndEdit()
            Next
         Next
         If form.NotNull AndAlso Not form.Validate() Then
            System.Media.SystemSounds.Hand.Play()
            Return
         End If
         System.Media.SystemSounds.Asterisk.Play()
         If checkForChanges AndAlso Not HasChanges.Invoke() Then Return
         _Save(tbls)
      End Sub

      Private Sub _HandleFormClosing(ByVal sender As Object, ByVal e As CancelEventArgs)
         If e.Cancel Then Return
         Dim frm = DirectCast(sender, Form)
         If Not HasChanges.Invoke() Then Return
         Select Case MessageBox.Show(frm, "Änderungen speichern?", "Das Ende ist nahe", MessageBoxButtons.YesNoCancel)
            Case Windows.Forms.DialogResult.Yes
               Save(frm, False)
            Case Windows.Forms.DialogResult.Cancel
               e.Cancel = True
               Return
         End Select
         e.Cancel = False
      End Sub

   End Class

End Namespace
