﻿Imports System.Runtime.CompilerServices
Imports System.Drawing
Imports System.Reflection
Imports System.ComponentModel

Namespace System.Windows.Forms

   Public Module DataGridViewX

      Private ReadOnly AutoHeightGrids As New ComponentProperty( _
         Of DataGridView, GridAutoHeightBehavior)(Function(grd) New GridAutoHeightBehavior(grd))
      ''' <summary>
      ''' keeps grds height at RowCount * RowTemplate.Height. grd. must be docked top/bottom, maybe its ContainerControls as well, with AutoSize.True. Only useful with small amounts of data.
      ''' </summary>
      <Extension()> _
      Public Function InitAutoHeight(ByVal grd As DataGridView) As GridAutoHeightBehavior
         Return AutoHeightGrids(grd)
      End Function

      Private ReadOnly LockColumnGrids As New ComponentProperty( _
         Of DataGridView, GridLockColumnBehavior)(Function(grd) New GridLockColumnBehavior(grd))
      ''' <summary>
      ''' makes shure you can't enter a readonly Column at all. Note: One of the grids Columns must be writeable
      ''' </summary>
      <Extension()> _
      Public Function InitLockColumns(ByVal grd As DataGridView) As GridLockColumnBehavior
         Return LockColumnGrids(grd)
      End Function

      ''' <summary>
      ''' creates, configures and inserts a DataGridViewComboBoxColumn to display Enum-Values. The Enum-Values become persisted in a DataProperty of Type Integer. So make sure that there is one. The previous DataGridViewColumn will be replaced, but some of its Properties will be assigned to the new one.
      ''' </summary>
      <Extension()> _
      Public Function CreateEnumCombo(Of T As Structure)(ByVal oldDGVColumn As DataGridViewColumn) _
            As DataGridViewComboBoxColumn
         CreateEnumCombo = CreateCombo( _
            oldDGVColumn.DataGridView, oldDGVColumn.DataPropertyName, GetType(T))
         ReplaceCombo(oldDGVColumn, CreateEnumCombo)
      End Function

      ''' <summary>
      ''' creates, configures and inserts a DataGridViewComboBoxColumn to display Enum-Values. The Enum-Values become persisted in a DataProperty of Type Integer. So make sure that there is one. If there is exactly one previous DataGridViewColumn for that dataProperty, it will be replaced, but some of its Properties will be assigned to the new one.
      ''' </summary>
      <Extension()> _
      Public Function CreateEnumCombo(Of T As Structure)(ByVal dgv As DataGridView, _
            Optional ByVal dataPropertyName As String = Nothing) As DataGridViewComboBoxColumn
         Dim tp = GetType(T)
         If dataPropertyName.Null Then dataPropertyName = tp.Name
         CreateEnumCombo = CreateCombo(dgv, dataPropertyName, tp)
         Dim prevCols = dgv.Columns.Cast(Of DataGridViewColumn) _
            .Where(Function(c) c.DataPropertyName = dataPropertyName).ToArray
         If prevCols.Length = 1 Then ReplaceCombo(prevCols(0), CreateEnumCombo)
      End Function

      Private Sub ReplaceCombo(ByVal oldCol As DataGridViewColumn, ByVal newCol As DataGridViewComboBoxColumn)
         With oldCol
            newCol.AutoSizeMode = .AutoSizeMode
            newCol.ContextMenuStrip = .ContextMenuStrip
            newCol.DisplayIndex = .DisplayIndex
            newCol.DividerWidth = .DividerWidth
            newCol.FillWeight = .FillWeight
            newCol.Frozen = .Frozen
            newCol.HeaderText = .HeaderText
            newCol.MinimumWidth = .MinimumWidth
            newCol.ReadOnly = .ReadOnly
            If .ReadOnly Then newCol.DisplayStyle = DataGridViewComboBoxDisplayStyle.Nothing
            newCol.Resizable = .Resizable
            newCol.ToolTipText = .ToolTipText
            newCol.Width = .Width
            .DataGridView.Columns.Insert(.Index, newCol)
            .DataGridView.Columns.Remove(oldCol)
            newCol.Name = .Name
         End With
      End Sub

      ''' <summary>
      ''' creates and configures a DataGridViewComboBoxColumn to display Enum-Values. The Enum-Values become persisted in a DataProperty of Type Integer. So make sure that there is one.
      ''' </summary>
      Private Function CreateCombo(ByVal dgv As DataGridView, ByVal dataPropertyName As String, _
            ByVal tpEnum As Type) As DataGridViewComboBoxColumn
         If Not tpEnum.IsEnum Then
#If DEBUG Then
            Stop
#End If
            Throw New ArgumentException("the given Enum-Type is no Enum-Type", "tpEnum")
         End If
         Dim src = TryCast(dgv.DataSource, ITypedList)
         If src.Null Then
#If DEBUG Then
            Stop
#End If
            Throw New ArgumentException("the DataSource of the given DataGridView is no ITypedList", _
               "dgv.DataSource")
         End If
         If Not src.GetItemProperties(Nothing) _
               .Cast(Of PropertyDescriptor).Any(Function(pd) _
               pd.Name = dataPropertyName _
               AndAlso pd.PropertyType.Equals(GetType(Integer))) Then
            Dim errMsg = "couldn't find a Property '".And( _
               src.GetListName(Nothing), ".", dataPropertyName, "' with DataType Integer")
#If DEBUG Then
            Stop
#End If
            Throw New ArgumentException(errMsg, "dataPropertyName")
         End If
         CreateCombo = New DataGridViewComboBoxColumn
         With CreateCombo
            .DataPropertyName = dataPropertyName
            .DisplayMember = "Enm"
            .ValueMember = "Int"
            .DataSource = [Enum].GetValues(tpEnum).Cast(Of Object) _
               .Select(Function(Enm) _
               New With {.Enm = Enm.ToString, .Int = DirectCast(Enm, Integer)}).ToArray
         End With
      End Function

      <Extension()> _
      Public Function DataRow(Of T As DataRow)(ByVal grd As DataGridView, ByVal rowIndex As Integer) As T
         Return DirectCast(DirectCast(grd.Rows(rowIndex).DataBoundItem, DataRowView).Row, T)
      End Function

   End Module

End Namespace
