Imports System.Reflection
Imports System.Collections.Generic

Namespace System

   <AttributeUsage(AttributeTargets.Field)> _
   Public Class DisposableAttribute : Inherits Attribute

      Private Shared _MyType As Type = GetType(DisposableAttribute)

      Private Shared _AttributedFields As New Dictionary(Of Type, FieldInfo())

      Public Shared Sub DisposeFields(ByVal itm As Object)
         DisposeFields(itm.GetType, itm)
      End Sub

      ''' <exception cref="System.InvalidCastException">
      ''' wenn eines der als Disposable attributierten Felder von tp gar nicht IDisposable implementiert
      ''' </exception>
      ''' <param name="tp">der Typ, dessen Felder berprft werden</param>
      ''' <param name="itm">ein Instanz dieses Typs</param>
      Public Shared Sub DisposeFields(ByVal tp As Type, ByVal itm As Object)
         Dim fieldInfos As FieldInfo() = Nothing
         If Not _AttributedFields.TryGetValue(tp, fieldInfos) Then
            'wenn der Typ tp noch unbekannt ist, einen Eintrag tp / fieldInfos anlegen
            'von allen FieldInfos des Typs die mit <Disposable> attributierten aufsammeln
            Dim flds As FieldInfo() = itm.GetType.GetFields( _
                  BindingFlags.Instance Or BindingFlags.GetField Or BindingFlags.NonPublic Or BindingFlags.Public)
            Dim i As Integer
            For i = 0 To flds.Length - 1
               If Not flds(i).IsDefined(DisposableAttribute._MyType, False) Then Exit For
            Next
            For ii As Integer = i + 1 To flds.Length - 1
               If flds(ii).IsDefined(DisposableAttribute._MyType, False) Then
                  flds(i) = flds(ii)
                  i += 1
               End If
            Next
            ReDim fieldInfos(i - 1)
            Array.Copy(flds, fieldInfos, i)
            _AttributedFields.Add(tp, fieldInfos)
         End If
         'alle Werte der fieldInfos disposen
         For Each fld As FieldInfo In fieldInfos
            Dim fieldValue As Object = fld.GetValue(itm)
            If fieldValue IsNot Nothing Then
               Debug.WriteLine("disposing " & fld.Name)
               DirectCast(fieldValue, IDisposable).Dispose()
            End If
         Next
      End Sub

   End Class

End Namespace