Imports System.Collections.ObjectModel
Imports System.Diagnostics

#If NoCompile Then
   'Bestimmt im Haltemodus anzuzeigende Member
   <System.Diagnostics.DebuggerDisplay("Value = {_Value}")> _

   'versteckt Member vor Anzeige
   <DebuggerBrowsable(DebuggerBrowsableState.Never)> _

#End If

Public Class Tree(Of T) : Inherits Collection(Of Tree(Of T))

   Protected _Value As T
   Protected _Parent As Tree(Of T)

   Public Sub New(ByVal Value As T)
      Me._Value = Value
   End Sub

   Public Property Value() As T
      Get
         Return _Value
      End Get
      Set(ByVal value As T)
         _Value = value
      End Set
   End Property

   Public ReadOnly Property Parent() As Tree(Of T)
      Get
         Return _Parent
      End Get
   End Property

   Protected Overrides Sub ClearItems()
      For Each Tr As Tree(Of T) In Me
         Tr._Parent = Nothing
      Next
      MyBase.ClearItems()
   End Sub

   Protected Overrides Sub InsertItem(ByVal index As Integer, ByVal item As Tree(Of T))
      item._Parent = Me
      MyBase.InsertItem(index, item)
   End Sub

   Protected Overrides Sub RemoveItem(ByVal index As Integer)
      Me(index)._Parent = Nothing
      MyBase.RemoveItem(index)
   End Sub

   Protected Overrides Sub SetItem(ByVal index As Integer, ByVal item As Tree(Of T))
      Me(index)._Parent = Nothing
      item._Parent = Me
      MyBase.SetItem(index, item)
   End Sub

   Public Overrides Function ToString() As String
      Return _Value.ToString()
   End Function

   Private Overloads ReadOnly Property Count() As String
      Get
         'Debugger-Verarsche: der zeigt immer Count an, sonst nix. So hat man die wichtigestn 
         'Infos, und kann den Tree echt treemig aufklappen, auf engstem Raum
         Return String.Format("{0}"" - ""{1}", MyBase.Count, _Value)
      End Get
   End Property

   Public Overloads Function Add(ByVal Value As T) As Tree(Of T)
      Add = New Tree(Of T)(Value)
      MyBase.Add(Add)
   End Function

   Public Function ChildEnumerating() As HEnumerating
      Return New HEnumerating(Me)
   End Function

   Public Sub Populate( _
         ByVal OtherTrees As IEnumerable, _
         ByVal GetChilds As Func(Of T, IEnumerable))
      For Each Other As T In OtherTrees
         Dim ndNew = Me.Add(Other)
         ndNew.Populate(GetChilds(Other), GetChilds)
      Next
   End Sub

   Public Sub Populate(Of T2)( _
         ByVal OtherTrees As IEnumerable, _
         ByVal GetChilds As Func(Of T2, IEnumerable), _
         ByVal GetValue As Func(Of T2, T))
      For Each Other As T2 In OtherTrees
         Dim ndNew = Me.Add(GetValue(Other))
         ndNew.Populate(GetChilds(Other), GetChilds, GetValue)
      Next
   End Sub

   Public Function ToList() As List(Of Tree(Of T))
      ToList = New List(Of Tree(Of T))(Me)
      For Each Nd In Me
         Nd.FillList(ToList)
      Next
   End Function

   Private Sub FillList(ByVal Lst As List(Of Tree(Of T)))
      Lst.AddRange(Me)
      For Each Nd In Me
         Nd.FillList(Lst)
      Next
   End Sub

   Public Function RootEnumerating() As HEnumerating
      'um Root mit zu enumerieren wirds als einziges Element in ein Array gesteckt. Da Array(Of Tree(Of T)
      '!) ebenso wie List(Of Tree(Of T)) IEnumerable(Of Tree(Of T)) implementiert, merkt der Enumerating-
      '!Konstruktor den Unterschied gar nicht
      Return New HEnumerating(New Tree(Of T)() {Me})
   End Function

   Public Function GetNodeCount() As Integer
      GetNodeCount = MyBase.Count
      For Each Nd As Tree(Of T) In Me
         GetNodeCount += Nd.GetNodeCount
      Next
   End Function

End Class